/*
  Apery, a USI shogi playing engine derived from Stockfish, a UCI chess playing engine.
  Copyright (C) 2004-2008 Tord Romstad (Glaurung author)
  Copyright (C) 2008-2015 Marco Costalba, Joona Kiiski, Tord Romstad
  Copyright (C) 2015-2016 Marco Costalba, Joona Kiiski, Gary Linscott, Tord Romstad
  Copyright (C) 2011-2016 Hiraoka Takuya

  Apery is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Apery is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "common.hpp"
#include "bitboard.hpp"
#include "init.hpp"
#include "position.hpp"
#include "usi.hpp"
#include "thread.hpp"
#include "tt.hpp"
#include "search.hpp"

#if defined FIND_MAGIC
// Magic Bitboard の Magic Number を求める為のソフト
int main() {
    u64 RookMagic[SquareNum];
    u64 BishopMagic[SquareNum];

    std::cout << "const u64 RookMagic[81] = {" << std::endl;
    for (Square sq = SQ11; sq < SquareNum; ++sq) {
        RookMagic[sq] = findMagic(sq, false);
        std::cout << "\tUINT64_C(0x" << std::hex << RookMagic[sq] << ")," << std::endl;
    }
    std::cout << "};\n" << std::endl;

    std::cout << "const u64 BishopMagic[81] = {" << std::endl;
    for (Square sq = SQ11; sq < SquareNum; ++sq) {
        BishopMagic[sq] = findMagic(sq, true);
        std::cout << "\tUINT64_C(0x" << std::hex << BishopMagic[sq] << ")," << std::endl;
    }
    std::cout << "};\n" << std::endl;

    return 0;
}

#else
// 将棋を指すソフト
int main(int argc, char* argv[]) {
    initTable();
    Position::initZobrist();
    HuffmanCodedPos::init();
    auto s = std::unique_ptr<Searcher>(new Searcher);
    s->init();
    s->doUSICommandLoop(argc, argv);
    s->threads.exit();
}

#endif
